==> tutti-ui-swing-2.5-sources.jar: fr/ifremer/tutti/ui/swing/content/operation/catches/SampleCategoryComponent.java <==
package fr.ifremer.tutti.ui.swing.content.operation.catches;

/*
 * #%L
 * Tutti :: UI
 * $Id: SampleCategoryComponent.java 1153 2013-07-24 17:35:44Z tchemit $
 * $HeadURL: http://svn.forge.codelutin.com/svn/tutti/tags/tutti-2.5/tutti-ui-swing/src/main/java/fr/ifremer/tutti/ui/swing/content/operation/catches/SampleCategoryComponent.java $
 * %%
 * Copyright (C) 2012 Ifremer
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the 
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public 
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-3.0.html>.
 * #L%
 */

import fr.ifremer.tutti.persistence.entities.data.SampleCategory;
import fr.ifremer.tutti.ui.swing.util.TuttiUIUtil;
import jaxx.runtime.JAXXUtil;
import jaxx.runtime.swing.editor.NumberEditor;
import org.nuiton.decorator.Decorator;

import javax.swing.AbstractCellEditor;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.event.AncestorEvent;
import javax.swing.event.AncestorListener;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.io.Serializable;

/**
 * To render and edit a {@link SampleCategory}.
 *
 * @author tchemit <chemit@codelutin.com>
 * @since 0.3
 */
public class SampleCategoryComponent {

    public static <C extends Serializable> TableCellRenderer newRender(
            TableCellRenderer renderer,
            Decorator<C> decorator,
            Color computedDataColor) {
        return new SampleCategoryRenderer<C>(renderer, decorator, computedDataColor);
    }

    public static <C extends Serializable> TableCellEditor newEditor(Decorator<C> decorator) {
        return new SampleCategoryEditor<C>(decorator);
    }

    /**
     * SampleCategory cell editor.
     *
     * @author tchemit <chemit@codelutin.com>
     * @since 0.3
     */
    public static class SampleCategoryEditor<C extends Serializable> extends AbstractCellEditor
            implements TableCellEditor, FocusListener, AncestorListener {

        private static final long serialVersionUID = 1L;

        protected final NumberEditor numberEditor;

        protected final JPanel editor;

        protected final JLabel editorLabel;

        protected final Decorator<C> categoryDecorator;

        public SampleCategoryEditor(Decorator<C> categoryDecorator) {
            this.categoryDecorator = categoryDecorator;
            numberEditor = new NumberEditor();
            numberEditor.getTextField().setHorizontalAlignment(SwingConstants.RIGHT);
            numberEditor.getTextField().setBorder(null);
            numberEditor.getTextField().addFocusListener(this);
            numberEditor.getTextField().addAncestorListener(this);
            numberEditor.setModelType(Float.class);
            numberEditor.setUseSign(false);
            numberEditor.init();

            editor = new JPanel(new BorderLayout());
            editor.add(BorderLayout.WEST, editorLabel = new JLabel());
            editor.add(BorderLayout.CENTER, numberEditor);
        }

        public NumberEditor getNumberEditor() {
            return numberEditor;
        }

        @Override
        public Component getTableCellEditorComponent(JTable table,
                                                     Object value,
                                                     boolean isSelected,
                                                     int row,
                                                     int column) {

            SampleCategory<C> sampleCategory = (SampleCategory<C>) value;

            C categoryValue = sampleCategory == null ? null : sampleCategory.getCategoryValue();
            Float number = sampleCategory == null ? null : sampleCategory.getCategoryWeight();

            numberEditor.setModel(number);

            // Check nullity and set the text that will be selected with the current value
            if (number == null) {
                numberEditor.setModelText("");
            } else {
                numberEditor.setModelText(String.valueOf(number));
            }

            String label = sampleCategory == null ? "-" :
                           categoryDecorator.toString(categoryValue);

            editorLabel.setText(label);
            return editor;
        }

        @Override
        public Float getCellEditorValue() {
            return (Float) numberEditor.getModel();
        }

        @Override
        public void focusGained(FocusEvent e) {
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    numberEditor.getTextField().requestFocus();
                    numberEditor.getTextField().selectAll();
                }
            });
        }

        @Override
        public void focusLost(FocusEvent e) {
        }

        @Override
        public void ancestorAdded(AncestorEvent event) {
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    numberEditor.getTextField().requestFocus();
                    numberEditor.getTextField().selectAll();
                }
            });
        }

        @Override
        public void ancestorRemoved(AncestorEvent event) {
        }

        @Override
        public void ancestorMoved(AncestorEvent event) {
        }

        @Override
        public boolean stopCellEditing() {
            boolean result = super.stopCellEditing();
            // Reset previous data to avoid keeping it on other cell edition
            if (result) {
                resetEditor();
            }
            return result;
        }

        @Override
        public void cancelCellEditing() {
            resetEditor();
            super.cancelCellEditing();
        }

        protected void resetEditor() {
            numberEditor.setModel(null);
            // Use empty string, otherwise there is a NPE in NumberEditorHandler
            numberEditor.setModelText("");
            editorLabel.setText("-");
        }
    }

    /**
     * To render a {@link SampleCategory} in a table cell.
     *
     * @author tchemit <chemit@codelutin.com>
     * @since 0.3
     */
    public static class SampleCategoryRenderer<C extends Serializable> implements TableCellRenderer {

        protected final TableCellRenderer delegate;

        protected final Decorator<C> categoryDecorator;

        protected final Color computedWeightColor;

        public SampleCategoryRenderer(TableCellRenderer delegate,
                                      Decorator<C> categoryDecorator,
                                      Color computedWeightColor) {
            this.delegate = delegate;
            this.categoryDecorator = categoryDecorator;
            this.computedWeightColor = computedWeightColor;
        }

        @Override
        public Component getTableCellRendererComponent(JTable table,
                                                       Object value,
                                                       boolean isSelected,
                                                       boolean hasFocus,
                                                       int row,
                                                       int column) {

            SampleCategory<C> sampleCategory = (SampleCategory<C>) value;

            String text = null;
            if (sampleCategory != null) {
                C categoryValue = sampleCategory.getCategoryValue();
                if (categoryValue != null) {
                    Float number = sampleCategory.getCategoryWeight();
                    Float computedNumber = sampleCategory.getComputedWeight();

                    text = "<html><span style='white-space: nowrap;'>" + categoryDecorator.toString(categoryValue) + " /";

                    if (sampleCategory.isSubSample()) {
                        text += "/";
                    }
                    text += " ";

                    if (number != null) {
                        text += JAXXUtil.getStringValue(number);

                    } else if (computedNumber != null) {
                        if (sampleCategory.hasOnlyOneFrequency()) {
                            text += TuttiUIUtil.getWeightStringValue(computedNumber);

                        } else {
                            String color = Integer.toHexString(computedWeightColor.getRGB()).substring(2);
                            text += "<em style='color: #" + color + "'>" +
                                    TuttiUIUtil.getWeightStringValue(computedNumber) + "</em>";
                        }
                    } else {
                        text += "-";
                    }

                    text += "</span></html>";
                }
            }

            JComponent result =
                    (JComponent) delegate.getTableCellRendererComponent(table,
                                                                        text,
                                                                        isSelected,
                                                                        hasFocus,
                                                                        row,
                                                                        column);
            result.setToolTipText(text);

            return result;
        }
    }
}

==> tutti-ui-swing-2.5.1-sources.jar: fr/ifremer/tutti/ui/swing/content/operation/catches/SampleCategoryComponent.java <==
package fr.ifremer.tutti.ui.swing.content.operation.catches;

/*
 * #%L
 * Tutti :: UI
 * $Id: SampleCategoryComponent.java 1153 2013-07-24 17:35:44Z tchemit $
 * $HeadURL: http://svn.forge.codelutin.com/svn/tutti/tags/tutti-2.5.1/tutti-ui-swing/src/main/java/fr/ifremer/tutti/ui/swing/content/operation/catches/SampleCategoryComponent.java $
 * %%
 * Copyright (C) 2012 Ifremer
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the 
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public 
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-3.0.html>.
 * #L%
 */

import fr.ifremer.tutti.persistence.entities.data.SampleCategory;
import fr.ifremer.tutti.ui.swing.util.TuttiUIUtil;
import jaxx.runtime.JAXXUtil;
import jaxx.runtime.swing.editor.NumberEditor;
import org.nuiton.decorator.Decorator;

import javax.swing.AbstractCellEditor;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.event.AncestorEvent;
import javax.swing.event.AncestorListener;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.io.Serializable;

/**
 * To render and edit a {@link SampleCategory}.
 *
 * @author tchemit <chemit@codelutin.com>
 * @since 0.3
 */
public class SampleCategoryComponent {

    public static <C extends Serializable> TableCellRenderer newRender(
            TableCellRenderer renderer,
            Decorator<C> decorator,
            Color computedDataColor) {
        return new SampleCategoryRenderer<C>(renderer, decorator, computedDataColor);
    }

    public static <C extends Serializable> TableCellEditor newEditor(Decorator<C> decorator) {
        return new SampleCategoryEditor<C>(decorator);
    }

    /**
     * SampleCategory cell editor.
     *
     * @author tchemit <chemit@codelutin.com>
     * @since 0.3
     */
    public static class SampleCategoryEditor<C extends Serializable> extends AbstractCellEditor
            implements TableCellEditor, FocusListener, AncestorListener {

        private static final long serialVersionUID = 1L;

        protected final NumberEditor numberEditor;

        protected final JPanel editor;

        protected final JLabel editorLabel;

        protected final Decorator<C> categoryDecorator;

        public SampleCategoryEditor(Decorator<C> categoryDecorator) {
            this.categoryDecorator = categoryDecorator;
            numberEditor = new NumberEditor();
            numberEditor.getTextField().setHorizontalAlignment(SwingConstants.RIGHT);
            numberEditor.getTextField().setBorder(null);
            numberEditor.getTextField().addFocusListener(this);
            numberEditor.getTextField().addAncestorListener(this);
            numberEditor.setModelType(Float.class);
            numberEditor.setUseSign(false);
            numberEditor.init();

            editor = new JPanel(new BorderLayout());
            editor.add(BorderLayout.WEST, editorLabel = new JLabel());
            editor.add(BorderLayout.CENTER, numberEditor);
        }

        public NumberEditor getNumberEditor() {
            return numberEditor;
        }

        @Override
        public Component getTableCellEditorComponent(JTable table,
                                                     Object value,
                                                     boolean isSelected,
                                                     int row,
                                                     int column) {

            SampleCategory<C> sampleCategory = (SampleCategory<C>) value;

            C categoryValue = sampleCategory == null ? null : sampleCategory.getCategoryValue();
            Float number = sampleCategory == null ? null : sampleCategory.getCategoryWeight();

            numberEditor.setModel(number);

            // Check nullity and set the text that will be selected with the current value
            if (number == null) {
                numberEditor.setModelText("");
            } else {
                numberEditor.setModelText(String.valueOf(number));
            }

            String label = sampleCategory == null ? "-" :
                           categoryDecorator.toString(categoryValue);

            editorLabel.setText(label);
            return editor;
        }

        @Override
        public Float getCellEditorValue() {
            return (Float) numberEditor.getModel();
        }

        @Override
        public void focusGained(FocusEvent e) {
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    numberEditor.getTextField().requestFocus();
                    numberEditor.getTextField().selectAll();
                }
            });
        }

        @Override
        public void focusLost(FocusEvent e) {
        }

        @Override
        public void ancestorAdded(AncestorEvent event) {
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    numberEditor.getTextField().requestFocus();
                    numberEditor.getTextField().selectAll();
                }
            });
        }

        @Override
        public void ancestorRemoved(AncestorEvent event) {
        }

        @Override
        public void ancestorMoved(AncestorEvent event) {
        }

        @Override
        public boolean stopCellEditing() {
            boolean result = super.stopCellEditing();
            // Reset previous data to avoid keeping it on other cell edition
            if (result) {
                resetEditor();
            }
            return result;
        }

        @Override
        public void cancelCellEditing() {
            resetEditor();
            super.cancelCellEditing();
        }

        protected void resetEditor() {
            numberEditor.setModel(null);
            // Use empty string, otherwise there is a NPE in NumberEditorHandler
            numberEditor.setModelText("");
            editorLabel.setText("-");
        }
    }

    /**
     * To render a {@link SampleCategory} in a table cell.
     *
     * @author tchemit <chemit@codelutin.com>
     * @since 0.3
     */
    public static class SampleCategoryRenderer<C extends Serializable> implements TableCellRenderer {

        protected final TableCellRenderer delegate;

        protected final Decorator<C> categoryDecorator;

        protected final Color computedWeightColor;

        public SampleCategoryRenderer(TableCellRenderer delegate,
                                      Decorator<C> categoryDecorator,
                                      Color computedWeightColor) {
            this.delegate = delegate;
            this.categoryDecorator = categoryDecorator;
            this.computedWeightColor = computedWeightColor;
        }

        @Override
        public Component getTableCellRendererComponent(JTable table,
                                                       Object value,
                                                       boolean isSelected,
                                                       boolean hasFocus,
                                                       int row,
                                                       int column) {

            SampleCategory<C> sampleCategory = (SampleCategory<C>) value;

            String text = null;
            if (sampleCategory != null) {
                C categoryValue = sampleCategory.getCategoryValue();
                if (categoryValue != null) {
                    Float number = sampleCategory.getCategoryWeight();
                    Float computedNumber = sampleCategory.getComputedWeight();

                    text = "<html><span style='white-space: nowrap;'>" + categoryDecorator.toString(categoryValue) + " /";

                    if (sampleCategory.isSubSample()) {
                        text += "/";
                    }
                    text += " ";

                    if (number != null) {
                        text += JAXXUtil.getStringValue(number);

                    } else if (computedNumber != null) {
                        if (sampleCategory.hasOnlyOneFrequency()) {
                            text += TuttiUIUtil.getWeightStringValue(computedNumber);

                        } else {
                            String color = Integer.toHexString(computedWeightColor.getRGB()).substring(2);
                            text += "<em style='color: #" + color + "'>" +
                                    TuttiUIUtil.getWeightStringValue(computedNumber) + "</em>";
                        }
                    } else {
                        text += "-";
                    }

                    text += "</span></html>";
                }
            }

            JComponent result =
                    (JComponent) delegate.getTableCellRendererComponent(table,
                                                                        text,
                                                                        isSelected,
                                                                        hasFocus,
                                                                        row,
                                                                        column);
            result.setToolTipText(text);

            return result;
        }
    }
}
